#========================================================================================
# package      DE_EPAGES::Order::t::unit::LineItemContainerData
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Order::API::LineItemContainerData
#========================================================================================
package DE_EPAGES::Order::t::unit::LineItemContainerData;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Order::API::LineItemContainerData;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::Mockery;
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_getLineItemContainerData_NormalProduct();
    $self->test_getLineItemContainerData_DownloadableProduct();

    $self->test_getShippingData_NormalProduct();
    $self->test_getShippingData_DownloadableProduct();

    $self->test_getPaymentData();

    done_testing();
}



sub test_getLineItemContainerData_NormalProduct {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createObjects_NormalProduct();
    my $LineItemContainerData = $Mockery->get('SUT');

    my $LanguageID = '2';
    my $LocaleID = 'en_GB';
    my $hLineItemContainerData = $LineItemContainerData->getLineItemContainerData($LanguageID, $LocaleID);

    my $hExpectedLineItemContainerData = $self->_createExpectedLineItemContainerData();
    is_deeply($hLineItemContainerData, $hExpectedLineItemContainerData, "$SubTestName - test if expected LineItemContainer data was returned.");
}



sub test_getLineItemContainerData_DownloadableProduct {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createObjects_DownloadableProduct();
    my $LineItemContainerData = $Mockery->get('SUT');

    my $LanguageID = '2';
    my $LocaleID = 'en_GB';
    my $hLineItemContainerData = $LineItemContainerData->getLineItemContainerData($LanguageID, $LocaleID);

    my $hExpectedLineItemContainerData = $self->_createExpectedLineItemContainerData_DownloadableProduct();
    is_deeply($hLineItemContainerData, $hExpectedLineItemContainerData, "$SubTestName - test if expected LineItemContainer data was returned.");
}



sub test_getShippingData_NormalProduct {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createObjects_NormalProduct();
    my $LineItemContainerData = $Mockery->get('SUT');

    my $LanguageID = '2';
    my $LocaleID = 'en_GB';
    my $hShippingData = $LineItemContainerData->getShippingData($LanguageID, $LocaleID);

    my $hExpectedShippingData = $self->_createExpectedShippingData();
    is_deeply($hShippingData, $hExpectedShippingData, "$SubTestName - test if expected shipping data was returned.");
}



sub test_getShippingData_DownloadableProduct {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createObjects_DownloadableProduct();
    my $LineItemContainerData = $Mockery->get('SUT');

    my $LanguageID = '2';
    my $LocaleID = 'en_GB';
    my $hShippingData = $LineItemContainerData->getShippingData($LanguageID, $LocaleID);

    my $hExpectedShippingData = undef;
    is_deeply($hShippingData, $hExpectedShippingData, "$SubTestName - test if expected shipping data was returned.");
}



sub test_getPaymentData {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createObjects_NormalProduct();
    my $LineItemContainerData = $Mockery->get('SUT');

    my $LanguageID = '2';
    my $LocaleID = 'en_GB';
    my $hPaymentData = $LineItemContainerData->getPaymentData($LanguageID, $LocaleID);

    my $hExpectedPaymentData = $self->_createExpectedPaymentData();
    is_deeply($hPaymentData, $hExpectedPaymentData, "$SubTestName - test if expected payment data was returned.");
}



sub _createObjects_NormalProduct {
    my $self = shift;
    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('LineItemContainer', ['Shop', 'LineItemShipping', 'LineItemPayment', 'LineItemProduct1'], sub {
        my ($Shop, $LineItemShipping, $LineItemPayment, $LineItemProduct1) = @_;
        return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer', {
            'Site' => $Shop,
            'CurrencyID' => '34',
            'TaxModel' => 'GROSS',
            'GrandTotal' => '123.45',
            'TotalBeforeTax' => '56',
            'TotalTax' => '23',
            'LineItemsSubTotal' => '55.2',
            'Shipping' => $LineItemShipping,
            'Payment' => $LineItemPayment,
            'LineItems' => [
                $LineItemProduct1
            ],
        });
    });

    $Mockery->extend($self->_createCommonMocks());

    return $Mockery->assemble();
}



sub _createObjects_DownloadableProduct {
    my $self = shift;
    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('LineItemContainer', ['Shop', 'LineItemPayment', 'LineItemProduct1'], sub {
        my ($Shop, $LineItemPayment, $LineItemProduct1) = @_;
        return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer', {
            'Site' => $Shop,
            'CurrencyID' => '34',
            'TaxModel' => 'GROSS',
            'GrandTotal' => '123.45',
            'TotalBeforeTax' => '56',
            'TotalTax' => '23',
            'LineItemsSubTotal' => '55.2',
            'Shipping' => undef, # downloadable products have no LineItemShipping!
            'Payment' => $LineItemPayment,
            'LineItems' => [
                $LineItemProduct1
            ],
        });
    });

    $Mockery->extend($self->_createCommonMocks());

    return $Mockery->assemble();
}



sub _createCommonMocks {
    my $self = shift;
    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('SUT', ['BasketOrOrder', 'Formatter', 'Context'], sub {
        my ($BasketOrOrder, $Formatter, $Context) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Order::API::LineItemContainerData', [$BasketOrOrder]);
        $Mockify->addMockWithReturnValueAndParameterCheck('_createFormatter', $Formatter, [{'object'=> 'DE_EPAGES::Shop::API::Object::Shop'}, 'string']);
        $Mockify->addMockWithReturnValueAndParameterCheck('_createShopRequestContext', $Context, [{'object'=> 'DE_EPAGES::Shop::API::Object::Shop'}]);
        return $Mockify->getMockObject();
    });

    $Mockery->register('BasketOrOrder', ['LineItemContainer', 'Shop'], sub {
        my ($LineItemContainer, $Shop) = @_;
        return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::Basket', {
            'Site' => $Shop,
            'LineItemContainer' => $LineItemContainer,
        });
    });

    $Mockery->register('Shop', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Shop::API::Object::Shop', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('siteLocale', 'en_GB', [{'string' => 'en_GB'}]);
        return $Mockify->getMockObject();
    });

    $Mockery->register('Formatter', ['Shop'], sub {
        my ($Shop) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Shop::API::Resource::Formatter', [$Shop, 'en_GB']);
        $Mockify->addMock('quantity', sub {
            my $MockedSelf = shift;
            my ($Quantity, $Unit) = @_;
            return $Quantity;
        });
        $Mockify->addMock('price', sub {
            my $MockedSelf = shift;
            my ($Price, $Currency, $TaxModel) = @_;
            return $Price;
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('LineItemShipping', ['ShippingMethod'], sub {
        my ($ShippingMethod) = @_;
        return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemShipping', {
            'LineItemPrice' => 2.40,
            'ShippingMethod' => $ShippingMethod,
        });
    });

    $Mockery->register('LineItemPayment', ['PaymentMethod'], sub {
        my ($PaymentMethod) = @_;
        return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemPayment', {
            'LineItemPrice' => 3.20,
            'PaymentMethod' => $PaymentMethod,
        });
    });

    $Mockery->register('LineItemProduct1', ['Product1', 'OrderUnit'], sub {
        my ($Product1, $OrderUnit) = @_;
        my $LineItemProduct = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Product::API::Object::LineItemProduct', {
            'LineItemPrice' => 32.99,
            'BasePrice' => 32.99,
            'Quantity' => 2,
            'OrderUnit' => $OrderUnit,
            'GUID' => '123-35-56562-324352',
            'SKU' => 'abc_12345346',
            'Name' => 'Product 1',
            'Product' => $Product1,
        });
        $LineItemProduct->mock('instanceOf', sub {return 1;});
        return $LineItemProduct;
    });

    $Mockery->register('Product1', [], sub {
        return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Product::API::Object::Product', {
            'GUID' => '245345-32534-324234',
            'ImageHotDeal' => 'hotdeal',
            'ImageLarge' => 'large',
            'ImageSmall' => 'small',
            'ImageMedium' => 'medium',
            'ImageThumbnail' => 'thumbnail',
            'WebPath' => '/web/path',
            'MainCharacteristics' => 'test product',
        });
    });

    $Mockery->register('OrderUnit', [], sub {
        return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::UnitOfMeasurement', {});
    });

    $Mockery->register('Context', ['Shop'], sub {
        my ($Shop) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Shop::API::Resource::ShopRequestContext', [$Shop]);
        $Mockify->addMockWithReturnValue('apiBaseUrl', 'http://baseurl.test.org');
        $Mockify->addMockWithReturnValue('imageBaseUrl', 'http://image.baseurl.test.org');
        return $Mockify->getMockObject();
    });

    $Mockery->register('ShippingMethod', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shipping::API::Object::ShippingMethod', {
            'GUID' => '6235-43324',
            'Name' => 'TestShipping',
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('PaymentMethod', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Payment::API::Object::PaymentMethod', {
            'GUID' => '7695-5473452',
            'Name' => 'TestPayment',
        });
        return $Mockify->getMockObject();
    });

    return $Mockery;
}



sub _createExpectedLineItemContainerData {
    return {
      'grandTotal' => '123.45',
      'productLineItems' => [
          {
            'images' => [
                  {
                    'url' => 'http://image.baseurl.test.org/web/path/hotdeal',
                    'classifier' => 'HotDeal'
                  },{
                    'url' => 'http://image.baseurl.test.org/web/path/large',
                    'classifier' => 'Large'
                  },{
                    'url' => 'http://image.baseurl.test.org/web/path/medium',
                    'classifier' => 'Medium'
                  },{
                    'url' => 'http://image.baseurl.test.org/web/path/small',
                    'classifier' => 'Small'
                  },{
                    'url' => 'http://image.baseurl.test.org/web/path/thumbnail',
                    'classifier' => 'Thumbnail'
                  },
                ],
            'sku' => 'abc_12345346',
            'quantity' => 2,
            'name' => 'Product 1',
            'lineItemPrice' => '32.99',
            'productId' => '245345-32534-324234',
            'lineItemId' => '123-35-56562-324352',
            'singleItemPrice' => '32.99',
            'essentialFeatures' => 'test product',
            'links' => [
                 {
                   'rel' => 'product',
                   'href' => 'http://baseurl.test.org/api/products/245345-32534-324234'
                 }
               ]
          }
        ],
      'totalBeforeTax' => '56',
      'totalTax' => '23',
      'lineItemsSubTotal' => '55.2',
      'shippingPrice' => '2.4',
    };
}



sub _createExpectedLineItemContainerData_DownloadableProduct {
    return {
      'grandTotal' => '123.45',
      'productLineItems' => [
          {
            'images' => [
                  {
                    'url' => 'http://image.baseurl.test.org/web/path/hotdeal',
                    'classifier' => 'HotDeal'
                  },{
                    'url' => 'http://image.baseurl.test.org/web/path/large',
                    'classifier' => 'Large'
                  },{
                    'url' => 'http://image.baseurl.test.org/web/path/medium',
                    'classifier' => 'Medium'
                  },{
                    'url' => 'http://image.baseurl.test.org/web/path/small',
                    'classifier' => 'Small'
                  },{
                    'url' => 'http://image.baseurl.test.org/web/path/thumbnail',
                    'classifier' => 'Thumbnail'
                  }
                ],
            'sku' => 'abc_12345346',
            'quantity' => 2,
            'name' => 'Product 1',
            'lineItemPrice' => '32.99',
            'productId' => '245345-32534-324234',
            'lineItemId' => '123-35-56562-324352',
            'singleItemPrice' => '32.99',
            'essentialFeatures' => 'test product',
            'links' => [
                 {
                   'rel' => 'product',
                   'href' => 'http://baseurl.test.org/api/products/245345-32534-324234'
                 }
               ]
          }
        ],
      'totalBeforeTax' => '56',
      'totalTax' => '23',
      'lineItemsSubTotal' => '55.2',
      'shippingPrice' => '0'
    };
}



sub _createExpectedShippingData {
    return {
        'shippingMethod' => {
            'name' => 'TestShipping',
            'id' => '6235-43324'
        },
        'price' => '2.4'
    };
}



sub _createExpectedPaymentData {
    return {
        'paymentMethod' => {
            'name' => 'TestPayment',
            'id' => '7695-5473452'
        },
        'price' => '3.2'
    };
}

__PACKAGE__->runScript();