#========================================================================================
# package      DE_EPAGES::Packlink::t::User_si
# base         DE_EPAGES::Shop::API::TestImportXML;
# state        public
#----------------------------------------------------------------------------------------
# description  System Integration Test for User
#========================================================================================
package DE_EPAGES::Packlink::t::User_si;

use base DE_EPAGES::Shop::API::TestImportXML;

use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::Packlink::API::User qw ( CreatePacklinkUser DeletePacklinkUser GetPacklinkAuthToken ExistsPacklinkUser );

use JSON;

use Test::More;

use strict;

sub test {
    my $self = shift;

    $self->test_User();

    done_testing();
}

sub test_User {
    my $self = shift;

    my $ShopPath = $self->shopPath();
    my $Shop     = LoadObjectByPath($ShopPath);

    $self->_test_create_packlink_user($Shop);
    $self->_test_get_packlink_auth_token($Shop);
    $self->_test_exists_packlink_user($Shop);
    $self->_test_delete_packlink_user($Shop);

}

sub _test_create_packlink_user {
    my $self = shift;
    my ($Shop) = @_;

    my $SubTestName = ( caller(0) )[3];

    my $User = CreatePacklinkUser( $Shop, 'password' );
    is( $User->alias, 'PacklinkConnector', "$SubTestName - Check that Packlink's user is called 'PacklinkConnector.'" );
    is( $User->get('IsActivated'), 1, "$SubTestName - Check that Packlink's user is activated." );

    DeletePacklinkUser($Shop);

    return;
}

sub _test_get_packlink_auth_token {
    my $self = shift;
    my ($Shop) = @_;

    my $SubTestName = ( caller(0) )[3];

    my $User = CreatePacklinkUser( $Shop, 'password' );
    my $AuthToken = GetPacklinkAuthToken( $User->get('Path'), 'password', 'http://example.epages.com/epages/shop.soap' );
    ok( $AuthToken =~ /^[0-9a-f]+$/i, "$SubTestName - Check if the AuthToken contains only hexadecimal characters" );

    DeletePacklinkUser($Shop);

    return;
}

sub _test_exists_packlink_user {
    my $self = shift;
    my ($Shop) = @_;

    my $SubTestName = ( caller(0) )[3];

    CreatePacklinkUser( $Shop, 'password' );
    ok( ExistsPacklinkUser($Shop), "$SubTestName - Check if Packlink's user exists." );

    DeletePacklinkUser($Shop);

    return;
}

sub _test_delete_packlink_user {
    my $self = shift;
    my ($Shop) = @_;

    my $SubTestName = ( caller(0) )[3];

    CreatePacklinkUser( $Shop, 'password' );
    DeletePacklinkUser($Shop);
    ok( !ExistsPacklinkUser($Shop), "$SubTestName - Check if Packlink's user can be deleted." );

    return;
}

sub shopAlias {
    return 'TestShopPacklink';
}

sub importXMLPath {
    return 'ShopXMLs/ShopWithPacklink.xml';
}

__PACKAGE__->runScript();

1;
