#========================================================================================
# package      DE_EPAGES::Packlink::t::unit::CredentialsPacklink
# base         DE_EPAGES::Core::API::Test
# state        public
# description  unit test for DE_EPAGES::Packlink::API::CredentialsPacklink
#========================================================================================
package DE_EPAGES::Packlink::t::unit::CredentialsPacklink;
use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::Packlink::API::Connector;
use DE_EPAGES::Packlink::API::CredentialsPacklink;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use Test::Exception;
use Test::MockObject::Extends;
use Test::More;
use DE_EPAGES::Core::API::Error qw ( Error );

use strict;

use constant PACKLINK_ID => '3453564646454';

#----------------------------------------------------------------------------------------
sub test {
    my $self = shift;
    $self->_test_createPacklinkCredentials_positive();
    $self->_test_createPacklinkCredentials_negative();
    done_testing();
}

#========================================================================================
sub _test_createPacklinkCredentials_positive {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $isShippingProviderMethod             = '1';
    my $GenerateError                        = '0';
    my $MockedShippingProviderMethodPacklink = $self->_createShippingProviderMethodPacklink();
    my $MockedShippingMethodPacklink         = $self->_createShippingMethodPacklink($isShippingProviderMethod,$MockedShippingProviderMethodPacklink);
    my $MockedForm                           = $self->_createForm();
    my $MockedShop                           = $self->_createShop();
    my $MockedCredentialsPacklink            = $self->_createCredentialsPacklink($MockedShippingMethodPacklink, $GenerateError);

    my $PacklinkIdBefore = $MockedShippingProviderMethodPacklink->get('PacklinkID');

    is( $PacklinkIdBefore, '', "$SubTestName - tests if PACKLINK_ID has not been set yet" );

    my $AreCredentialsCreated = $MockedCredentialsPacklink->createPacklinkCredentials( $MockedForm, $MockedShop );
    is ($AreCredentialsCreated, '1', "$SubTestName - Credentials have been successfully created" );
    my $PacklinkId = $MockedShippingProviderMethodPacklink->get('PacklinkID');
    is( $PacklinkId, PACKLINK_ID, "$SubTestName - tests if PACKLINK_ID has been set" );
}

#========================================================================================
sub _test_createPacklinkCredentials_negative {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $isShippingProviderMethod             = '1';
    my $GenerateError                        = '1';
    my $MockedShippingProviderMethodPacklink = $self->_createShippingProviderMethodPacklink();
    my $MockedShippingMethodPacklink         = $self->_createShippingMethodPacklink($isShippingProviderMethod,$MockedShippingProviderMethodPacklink);
    my $MockedForm                           = $self->_createForm();
    my $MockedShop                           = $self->_createShop();
    my $MockedCredentialsPacklink            = $self->_createCredentialsPacklink($MockedShippingMethodPacklink, $GenerateError);

    my $PacklinkIdBefore = $MockedShippingProviderMethodPacklink->get('PacklinkID');

    is( $PacklinkIdBefore, '', "$SubTestName - tests if PACKLINK_ID has not been set yet" );

    my $AreCredentialsCreated = $MockedCredentialsPacklink->createPacklinkCredentials( $MockedForm, $MockedShop );
    is ($AreCredentialsCreated, '0', "$SubTestName - Credentials have been successfully created" );
    my $PacklinkId = $MockedShippingProviderMethodPacklink->get('PacklinkID');
    is( $PacklinkId, PACKLINK_ID, "$SubTestName - tests if PACKLINK_ID has been set" );
}

#----------------------------------------------------------------------------------------
sub _createCredentialsPacklink {
    my $self = shift;
    my ( $MockedShippingMethodPacklink, $GenerateError ) = @_;

    my $CredentialsPacklink       = DE_EPAGES::Packlink::API::CredentialsPacklink->new($MockedShippingMethodPacklink);
    my $MockedCredentialsPacklink = Test::MockObject::Extends->new($CredentialsPacklink);

    $MockedCredentialsPacklink->mock(
        '_GenerateNewPassword',
        sub {
            shift;
            return '1234567890';
        }
    );

    $MockedCredentialsPacklink->mock(
        '_createConnector',
        sub {
            shift;
            my $MockedShippingProviderMethodPacklink = $self->_createShippingProviderMethodPacklink();
            return $self->_createConnector($MockedShippingProviderMethodPacklink);
        }
    );

    $MockedCredentialsPacklink->mock(
        '_CreatePacklinkUser',
        sub {
            shift;

            my $hAtributes = { 'PathString' => '/Shops/DemoShop/Users/PacklinkConnector', };

            return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Permission::API::Object::User', $hAtributes );
        }
    );

    $MockedCredentialsPacklink->mock(
        '_GetPacklinkAuthToken',
        sub {
            shift;
            return 'ddyhfrtfwrt3gwrt3gwrt3whh589789';
        }
    );

    $MockedCredentialsPacklink->mock(
        '_LoadRootObject',
        sub {
            shift;
            my $hAttributes = { 'PacklinkProviderKey' => 'gs45345sdgfs45345hsdq34578dfgd', };
            my $MockedSystem =
              DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::Site', $hAttributes );
            return $MockedSystem;
        }
    );

    $MockedCredentialsPacklink->mock(
        '_SendPacklinkAuthCredentials',
        sub {
            shift;
            if( $GenerateError ){
                return Error(' Error SendPacklinkAuthCredentials ');
            }
            return;
        }
    );

    $MockedCredentialsPacklink->mock(
        '_DeletePacklinkUser',
        sub {
            shift;
            return;
        }
    );
}

#----------------------------------------------------------------------------------------
sub _createShippingMethodPacklink {
    my $self = shift;
    my ($isShippingProviderMethod,$MockedShippingProviderMethodPacklink) = @_;

    my $MockedShippingMethodPacklink =
      DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Packlink::API::ShippingMethodPacklink');

    $MockedShippingMethodPacklink->mock(
        'getShippingProviderMethodPacklink',
        sub {
            shift;
            return $MockedShippingProviderMethodPacklink;
        }
    );

    return $MockedShippingMethodPacklink;
}

#----------------------------------------------------------------------------------------
sub _createShippingProviderMethodPacklink {
    my $self = shift;

    my $hAtributes = { 'ShippingFulfillmentType' => $self->_createShippingFulfillmentType(), 'PacklinkID' => '' };

    my $MockedShippingProviderMethodPacklink =
      DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Packlink::API::Object::ShippingProviderMethodPacklink',
        $hAtributes );

    return $MockedShippingProviderMethodPacklink;
}

#----------------------------------------------------------------------------------------
sub _createShippingFulfillmentType {
    my $self = shift;

    my $MockedShippingFulfillmentTypePacklink =
      DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Packlink::API::Object::ShippingFulfillmentTypePacklink');

    return $MockedShippingFulfillmentTypePacklink;
}

#----------------------------------------------------------------------------------------
sub _createForm {
    my $self = shift;

    my $MockedForm = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Presentation::API::Form');

    $MockedForm->mock(
        'value',
        sub {
            my $self     = shift;
            my ($hValue) = @_;
            my $packlinkID = PACKLINK_ID;
            return $packlinkID;

        }
    );

    return $MockedForm;
}

#----------------------------------------------------------------------------------------
sub _createShop {
    my $self = shift;

    my $MockedShop = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {'IsInternalTestShop' => '1'});

    return $MockedShop;
}

#----------------------------------------------------------------------------------------
sub _createConnector {
    my $self = shift;
    my ($MockedShippingMethodPacklink) = @_;

    my $Connector       = DE_EPAGES::Packlink::API::Connector->new($MockedShippingMethodPacklink);
    my $MockedConnector = Test::MockObject::Extends->new($Connector);

    $MockedConnector->mock(
        'SendPacklinkAuthCredentials',
        sub {
            my $self = shift;
            return;

        }
    );

    return $MockedConnector;
}

__PACKAGE__->runScript();

1;
