#========================================================================================
# package      DE_EPAGES::Packlink::t::unit::RegistrationValidation
# base         DE_EPAGES::Core::API::Test
# state        public
# description  Unit test for TLE/RegistrationValidation.t
#========================================================================================
package DE_EPAGES::Packlink::t::unit::RegistrationValidation;
use base DE_EPAGES::Core::API::Test;

use Test::More;
use Test::MockObject::Extends;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Packlink::API::TLE::RegistrationValidation;
use DE_EPAGES::Packlink::API::Object::ShippingProviderMethodPacklink;
use strict;

#----------------------------------------------------------------------------------------
sub test {
    my $self = shift;
    $self->_test_can_packlink_connect_to_shop_positive();
    $self->_test_can_packlink_connect_to_shop_negative();
    done_testing();
}

#----------------------------------------------------------------------------------------
sub _test_can_packlink_connect_to_shop_positive {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ShippingProviderMethodPacklinkMocked = $self->_createShippingProviderMethod();
    my $aParams = ['67890'];
    my $MockedRegistrationValidation = $self->_getMockedRegistrationValidation($ShippingProviderMethodPacklinkMocked, 1);
    my $IsConnectionSetupDone = $MockedRegistrationValidation->CAN_PACKLINK_CONNECT_TO_SHOP('', $aParams);

    is($IsConnectionSetupDone, 1, "$SubTestName - The connection is setup correctly.");
    return;
}

#----------------------------------------------------------------------------------------
sub _test_can_packlink_connect_to_shop_negative {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ShippingProviderMethodPacklinkMocked = $self->_createShippingProviderMethod();
    my $aParams = ['67890'];
    my $MockedRegistrationValidation = $self->_getMockedRegistrationValidation($ShippingProviderMethodPacklinkMocked, 0);
    my $IsConnectionSetupDone = $MockedRegistrationValidation->CAN_PACKLINK_CONNECT_TO_SHOP('', $aParams);

    is($IsConnectionSetupDone, 0, "$SubTestName - The connection is setup correctly.");
    return;
}

#WRAPPERS
#----------------------------------------------------------------------------------------
sub _createShippingProviderMethod {
    my $self = shift;
    my ($WithoutShippingProviderMethods) = @_;

    if ($WithoutShippingProviderMethods) {
        return [];
    }

    my $Attributes = {
        'Alias'     => 'Packlink',
        'PacklinkID'  => '67890',
    };
    my $ShippingProviderMethodPacklink =
        DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Packlink::API::Object::ShippingProviderMethodPacklink', $Attributes);
    my $ShippingProviderMethodPacklinkMocked = Test::MockObject::Extends->new($ShippingProviderMethodPacklink);

    $ShippingProviderMethodPacklinkMocked->mock('isUsable', sub {
        shift;
        return 1;
    });

    return $ShippingProviderMethodPacklinkMocked;
}

#----------------------------------------------------------------------------------------
sub _getMockedRegistrationValidation {
    my $self = shift;
    my ($ShippingProviderMethodPacklinkMocked, $IsPositive) = @_;

    my $RegistrationValidation = DE_EPAGES::Packlink::API::TLE::RegistrationValidation->new();
    my $RegistrationValidationMocked = Test::MockObject::Extends->new($RegistrationValidation);

    $RegistrationValidationMocked->mock('_LoadObject', sub {
        shift;
        return $ShippingProviderMethodPacklinkMocked;
    });

    $RegistrationValidationMocked->mock('_TestObject', sub {
        shift;
        return;
    });

    $RegistrationValidationMocked->mock('_createConnector', sub {
        shift;
        my $Attributes = {
            'Method' => $ShippingProviderMethodPacklinkMocked
        };
        my $Connector = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Packlink::API::Connector', $Attributes);
        my $ConnectorMocked = Test::MockObject::Extends->new($Connector);

        $ConnectorMocked->mock('isConnectionSetupDone', sub{
            shift;
            $IsPositive ? return 1 : return 0;
        });

        return $ConnectorMocked;
    });

    return $RegistrationValidationMocked;
}


__PACKAGE__->runScript();