#========================================================================================
# package      DE_EPAGES::Packlink::t::unit::User
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::Packlink::API::User
#========================================================================================
package DE_EPAGES::Packlink::t::unit::User;
use base DE_EPAGES::Core::API::Test;
use Test::More;
use Test::Exception;
use DE_EPAGES::Packlink::API::User;
use Encode qw ( decode );
use JSON;
use strict;

sub test {
    my $self = shift;
    $self->test_encrypt();
    done_testing();
}
## no critic (ProhibitLongLines)
sub test_encrypt {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PlaintextToken = encode_json({
        'user' => '/Shops/DemoShop/Users/PacklinkConecctor',
        'password' => 'somesecretPassword',
        'soapUrl' => 'http://host/epages/shop.soap'
    });

    pass('plaintext token: ' . $PlaintextToken);
    my $Encrypted = DE_EPAGES::Packlink::API::User::_Encrypt( $PlaintextToken, 'thisSecretKeyIs32CharactersLongg' );    ## no critic (ProtectPrivateSubs)
    cmp_ok( $Encrypted, 'ne', $PlaintextToken, "$SubTestName - not plaintext anymore: " . $Encrypted);
    is( _decrypt($Encrypted), $PlaintextToken, "$SubTestName - can decrypt back into plaintext");

    throws_ok
        sub { DE_EPAGES::Packlink::API::User::_Encrypt( $PlaintextToken, undef) }, ## no critic (ProtectPrivateSubs)
        qr/PARAMETER_MISSING.*Key/, 'encryption key is mandatory';
}

#--------------------------------------------------------------------------------------------------------------
sub _decrypt {
    my ($Encrypted) = @_;
    my $Key = 'thisSecretKeyIs32CharactersLongg';
    my $Cipher = Crypt::CBC->new(
        -key    => $Key,
        -cipher => 'Crypt::Rijndael',
        -iv          => '$KJh#(}q%etp^/r7',
        -literal_key => 1,
        -header      => 'none',
        -padding     => 'null',
        -keysize     => 32
    );

    my $Decrypted = $Cipher->decrypt_hex($Encrypted);
    return decode( 'utf8', $Decrypted );
}

## use critic (ProhibitLongLines)
__PACKAGE__->runScript();