package DE_EPAGES::Test::t::unit::UnitTest_Test;
use base DE_EPAGES::Core::API::Test;
use Test::More;
use Test::Exception;
use DE_EPAGES::Test::API::UnitTest::Test;
use strict;

sub test {
    my $self = shift;
    $self->test_ObjectGetter();
    $self->test_DirectMemoryUse();
    $self->test_MockmemoryUse();
    $self->test_MixedMockmemoryAndDirectMemory();
    $self->test_RethrowError();
    done_testing();
}

sub test_ObjectGetter {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $Test = DE_EPAGES::Test::API::UnitTest::Test->new();
    isa_ok(
        $Test->memory(),
        'DE_EPAGES::Test::Mock::MockMemory',
        "$SubTestName - test if no parameter are given, the return value is the MockMemory Object"
    );
}

sub test_DirectMemoryUse {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $Test = DE_EPAGES::Test::API::UnitTest::Test->new();
    is($Test->memory('key', 'value'), 'value', "$SubTestName - tests if the value can be set and return the value");
    is($Test->memory('key'),'value', "$SubTestName - tests if the value can taken from memory store");
}

sub test_MockmemoryUse {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $Test = DE_EPAGES::Test::API::UnitTest::Test->new();
    is($Test->memory()->add('key', 'value'), 'value', "$SubTestName - tests if the value can be set and return the value");
    is($Test->memory()->get('key'),'value', "$SubTestName - tests if the value can taken from memory store");
}

sub test_MixedMockmemoryAndDirectMemory {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $Test = DE_EPAGES::Test::API::UnitTest::Test->new();
    is($Test->memory()->add('KeySetViaAdd', 'addValue'), 'addValue', "$SubTestName - tests if the value can be set and return the value");
    is($Test->memory('KeySetViaAdd'),'addValue', "$SubTestName - tests if the value can taken from memory store");
    is($Test->memory('KeySetDirect', 'directValue'), 'directValue', "$SubTestName - tests if the value can be set and return the value");
    is($Test->memory()->get('KeySetDirect'),'directValue', "$SubTestName - tests if the value can taken from memory store");
}

sub test_RethrowError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $Test = DE_EPAGES::Test::API::UnitTest::Test->new();
    throws_ok( sub { $Test->memory('NotExistingKey') },
                   qr/Key named 'NotExistingKey' does not exist./,
                   "$SubTestName - tests if with direct rethrow the error"
    );
    throws_ok( sub { $Test->memory->get('OtherNotExistingKey') },
                   qr/Key named 'OtherNotExistingKey' does not exist./,
                   "$SubTestName - tests if with direct rethrow the error"
    );
}
__PACKAGE__->runScript();